@@ -22,6 +22,7 @@
 #include "ephy-about-handler.h"
 
 #include "ephy-embed-shell.h"
+#include "ephy-embed-prefs.h"
 #include "ephy-file-helpers.h"
 #include "ephy-smaps.h"
 #include "ephy-web-app-utils.h"
@@ -137,6 +138,13 @@ get_plugins_cb (WebKitWebContext *web_context,
   GString *data_str;
   gsize data_length;
   GList *plugin_list, *p;
+  WebKitWebViewGroup *web_view_group;
+  WebKitSettings *webkit_settings;
+  gboolean enabled;
+
+  web_view_group = ephy_embed_prefs_get_web_view_group ();
+  webkit_settings = webkit_web_view_group_get_settings (web_view_group);
+  enabled = webkit_settings_get_enable_plugins (webkit_settings);
 
   data_str = g_string_new ("<html>");
   g_string_append_printf (data_str, "<head><title>%s</title>"           \
@@ -144,7 +152,10 @@ get_plugins_cb (WebKitWebContext *web_context,
                           "<style type=\"text/css\">%s</style></head><body>",
                           _("Installed plugins"),
                           ephy_about_handler_get_style_sheet (about_request->handler));
-  g_string_append_printf (data_str, "<h1>%s</h1>", _("Installed plugins"));
+  g_string_append_printf (data_str, "<h1>%s</h1>", _("Plugins"));
+
+  if (!enabled)
+    g_string_append_printf (data_str, "<p><b>%s</b></p>", _("Plugins are disabled in the preferences"));
 
   plugin_list = webkit_web_context_get_plugins_finish (web_context, result, NULL);
   for (p = plugin_list; p; p = p->next) {
@@ -157,7 +168,7 @@ get_plugins_cb (WebKitWebContext *web_context,
                             "  <thead><tr><th>%s</th><th>%s</th><th>%s</th></tr></thead><tbody>",
                             webkit_plugin_get_name (plugin),
                             webkit_plugin_get_description (plugin),
-                            _("Enabled"), /*webkit_plugin_get_enabled (plugin)*/ TRUE ? _("Yes") : _("No"),
+                            _("Enabled"), /*webkit_plugin_get_enabled (plugin) && */ enabled ? _("Yes") : _("No"),
                             _("MIME type"), _("Description"), _("Suffixes"));
 
     mime_types = webkit_plugin_get_mime_info_list (plugin);
@@ -319,10 +330,10 @@ ephy_about_handler_handle_epiphany (EphyAboutHandler *handler,
   data = g_strdup_printf ("<html><head><title>%s</title>"
                           "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />"
                           "<style type=\"text/css\">%s</style></head>"
-                          "<body style=\"background: #3369FF; color: white; font-style: italic;\">"
+                          "<body class=\"epiphany-body\">"
                           "<div id=\"ephytext\">"
-                          "Il semble que la perfection soit atteinte non quand il n'y a plus rien à"
-                          " ajouter, mais quand il n'y a plus rien à retrancher."
+                          "“Il semble que la perfection soit atteinte non quand il n'y a plus rien à"
+                          " ajouter, mais quand il n'y a plus rien à retrancher.”"
                           "</div>"
                           "<div id=\"from\">"
                           "<!-- Terre des Hommes, III: L'Avion, p. 60 -->"
@@ -419,21 +430,25 @@ static gboolean
 ephy_about_handler_handle_incognito (EphyAboutHandler *handler,
                                      WebKitURISchemeRequest *request)
 {
-  const char *filename;
-  char *img_data = NULL, *img_data_base64 = NULL;
+  GBytes *img_data;
+  char *img_data_base64 = NULL;
   char *data;
-  gsize data_length;
+  GError *error;
 
   if (ephy_embed_shell_get_mode (ephy_embed_shell_get_default ()) != EPHY_EMBED_SHELL_MODE_INCOGNITO)
     return FALSE;
 
-  filename = ephy_file ("incognito.png");
-  if (filename) {
-    g_file_get_contents (filename, &img_data, &data_length, NULL);
-    img_data_base64 = g_base64_encode ((guchar*)img_data, data_length);
+  img_data = g_resources_lookup_data ("/org/gnome/epiphany/incognito.png", 0, &error);
+  if (img_data != NULL) {
+    img_data_base64 = g_base64_encode ((guchar*)g_bytes_get_data (img_data, NULL),
+                                       g_bytes_get_size (img_data));
+  } else {
+    g_debug ("%s", error->message);
+    g_error_free (error);
   }
 
   data = g_strdup_printf ("<html>\n"                                   \
+                          "<div dir=\"%s\">\n"                         \
                           "<head>\n"                                   \
                           "<title>%s</title>\n"                        \
                           "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />" \
@@ -447,7 +462,9 @@ ephy_about_handler_handle_incognito (EphyAboutHandler *handler,
                           "    </div>\n"                               \
                           "  </div>\n"                                 \
                           "</body>\n"                                  \
+                          "</div>\n"                                   \
                           "</html>\n",
+                          gtk_widget_get_default_direction () == GTK_TEXT_DIR_RTL ? "rtl" : "ltr",
                           _("Private Browsing"),
                           ephy_about_handler_get_style_sheet (handler),
                           img_data_base64 ? img_data_base64 : "",
@@ -457,7 +474,7 @@ ephy_about_handler_handle_incognito (EphyAboutHandler *handler,
                             "information will be cleared when you close the window."));
 
   g_free (img_data_base64);
-  g_free (img_data);
+  g_bytes_unref (img_data);
 
   ephy_about_handler_finish_request (request, data, -1);
 
