@@ -61,7 +61,6 @@ enum
   PROP_0,
   PROP_DOWNLOAD,
   PROP_DESTINATION,
-  PROP_SOURCE,
   PROP_ACTION,
   PROP_START_TIME,
   PROP_WINDOW,
@@ -93,9 +92,6 @@ ephy_download_get_property (GObject    *object,
     case PROP_DESTINATION:
       g_value_set_string (value, ephy_download_get_destination_uri (download));
       break;
-    case PROP_SOURCE:
-      g_value_set_string (value, ephy_download_get_source_uri (download));
-      break;
     case PROP_ACTION:
       g_value_set_enum (value, ephy_download_get_action (download));
       break;
@@ -131,7 +127,6 @@ ephy_download_set_property (GObject      *object,
       ephy_download_set_widget (download, g_value_get_object (value));
       break;
     case PROP_DOWNLOAD:
-    case PROP_SOURCE:
     case PROP_START_TIME:
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -143,9 +138,8 @@ ephy_download_set_property (GObject      *object,
  * ephy_download_get_content_type:
  * @download: an #EphyDownload
  *
- * Gets content-type information for @download. If the file is already
- * present on the filesystem and readable, uses GIO to get the
- * content-type. Otherwise it uses WebKit and Soup.
+ * Gets content-type information for @download. If the server didn't
+ * provide a content type, the destination file is queried.
  *
  * Returns: content-type for @download, must be freed with g_free()
  **/
@@ -153,40 +147,40 @@ char *
 ephy_download_get_content_type (EphyDownload *download)
 {
   WebKitURIResponse *response;
+  const char *destination_uri;
+  GFile *destination;
+  GFileInfo *info;
   char *content_type = NULL;
   GError *error = NULL;
 
-  if (download->priv->destination) {
-    GFile *destination;
-    GFileInfo *info;
-
-    destination = g_file_new_for_uri (download->priv->destination);
-    info = g_file_query_info (destination, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
-                              G_FILE_QUERY_INFO_NONE, NULL, &error);
-    if (info) {
-      content_type = g_strdup (g_file_info_get_content_type (info));
-      LOG ("ephy_download_get_content_type: GIO: %s", content_type);
-      g_object_unref (info);
-    } else {
-      LOG ("ephy_download_get_content_type: error getting file "
-           "content-type: %s", error->message);
-      g_error_free (error);
-    }
-
-    g_object_unref (destination);
-  }
-
-  if (content_type)
-    return content_type;
-
-  /* Fallback to Soup */
   response = webkit_download_get_response (download->priv->download);
-  if (response)
+  if (response) {
     content_type = g_strdup (webkit_uri_response_get_mime_type (response));
 
-  LOG ("ephy_download_get_content_type: Soup: %s", content_type);
+    LOG ("ephy_download_get_content_type: WebKit: %s", content_type);
 
-  LOG ("ephy_download_get_content_type: %s", content_type);
+    if (content_type)
+      return content_type;
+  }
+
+  destination_uri = webkit_download_get_destination (download->priv->download);
+  if (!destination_uri)
+    return NULL;
+
+  destination = g_file_new_for_uri (destination_uri);
+  info = g_file_query_info (destination, G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+                            G_FILE_QUERY_INFO_NONE, NULL, &error);
+  if (info) {
+    content_type = g_strdup (g_file_info_get_content_type (info));
+    LOG ("ephy_download_get_content_type: GIO: %s", content_type);
+    g_object_unref (info);
+  } else {
+    LOG ("ephy_download_get_content_type: error getting file "
+         "content-type: %s", error->message);
+    g_error_free (error);
+  }
+
+  g_object_unref (destination);
 
   return content_type;
 }
@@ -228,7 +222,7 @@ static const char*
 file_is_compressed (const char *filename)
 {
   int i;
-  static const char * const compression[] = {".gz", ".bz2", ".Z", ".lz", NULL};
+  static const char * const compression[] = {".gz", ".bz2", ".Z", ".lz", ".xz", NULL};
 
   for (i = 0; compression[i] != NULL; i++) {
     if (g_str_has_suffix (filename, compression[i]))
@@ -273,8 +267,8 @@ parse_extension (const char *filename)
   return strrchr ((last_separator) ? last_separator : filename, '.');
 }
 
-static char *
-define_destination_uri (EphyDownload *download, const char *suggested_filename)
+static gboolean
+set_destination_uri_for_suggested_filename (EphyDownload *download, const char *suggested_filename)
 {
   char *dest_dir;
   char *dest_name;
@@ -288,7 +282,7 @@ define_destination_uri (EphyDownload *download, const char *suggested_filename)
     g_critical ("Could not create downloads directory \"%s\": %s",
                 dest_dir, strerror (errno));
     g_free (dest_dir);
-    return NULL;
+    return FALSE;
   }
 
   if (suggested_filename != NULL) {
@@ -334,8 +328,10 @@ define_destination_uri (EphyDownload *download, const char *suggested_filename)
   g_free (destination_filename);
 
   g_assert (destination_uri);
+  webkit_download_set_destination (download->priv->download, destination_uri);
+  g_free (destination_uri);
 
-  return destination_uri;
+  return TRUE;
 }
 
 /**
@@ -350,35 +346,13 @@ void
 ephy_download_set_destination_uri (EphyDownload *download,
                                    const char *destination)
 {
-  EphyDownloadPrivate *priv;
-  char *scheme;
-
   g_return_if_fail (EPHY_IS_DOWNLOAD (download));
   g_return_if_fail (destination != NULL);
 
-  priv = download->priv;
-
-  scheme = g_uri_parse_scheme (destination);
-  g_return_if_fail (scheme != NULL);
-  g_free (scheme);
-
-  priv->destination = g_strdup (destination);
-
+  webkit_download_set_destination (download->priv->download, destination);
   g_object_notify (G_OBJECT (download), "destination");
 }
 
-/**
- * ephy_download_set_auto_destination:
- * @download: an #EphyDownload
- *
- * Tells @download to automatically determine a destination for itself.
- **/
-void
-ephy_download_set_auto_destination (EphyDownload *download)
-{
-  /* TODO: in WebKit2 priv->destination == NULL means auto_destination, remove this? */
-}
-
 /**
  * ephy_download_set_action:
  * @download: an #EphyDownload
@@ -484,23 +458,7 @@ ephy_download_get_destination_uri (EphyDownload *download)
 {
   g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), NULL);
 
-  return download->priv->destination;
-}
-
-/**
- * ephy_download_get_source_uri:
- * @download: an #EphyDownload
- *
- * Gets the source URI that this download is/will download.
- *
- * Returns: source URI.
- **/
-const char *
-ephy_download_get_source_uri (EphyDownload *download)
-{
-  g_return_val_if_fail (EPHY_IS_DOWNLOAD (download), NULL);
-
-  return download->priv->source;
+  return webkit_download_get_destination (download->priv->download);
 }
 
 /**
@@ -527,7 +485,7 @@ ephy_download_get_action (EphyDownload *download)
  * @download: an #EphyDownload
  *
  * Gets the time (returned by gtk_get_current_event_time ()) when @download was
- * started with ephy_download_start (). Defaults to 0.
+ * started. Defaults to 0.
  *
  * Returns: the time when @download was started.
  **/
@@ -539,26 +497,6 @@ ephy_download_get_start_time (EphyDownload *download)
   return download->priv->start_time;
 }
 
-/**
- * ephy_download_start:
- * @download: an #EphyDownload
- *
- * Starts the wrapped #WebKitDownload.
- **/
-void
-ephy_download_start (EphyDownload *download)
-{
-  EphyDownloadPrivate *priv;
-
-  g_return_if_fail (EPHY_IS_DOWNLOAD (download));
-
-  priv = download->priv;
-  priv->start_time = gtk_get_current_event_time ();
-
-  if (priv->destination == NULL)
-    ephy_download_set_auto_destination (download);
-}
-
 /**
  * ephy_download_cancel:
  * @download: an #EphyDownload
@@ -657,27 +595,9 @@ ephy_download_dispose (GObject *object)
     priv->widget = NULL;
   }
 
-  ephy_embed_shell_remove_download (ephy_embed_shell_get_default (), download);
-
   G_OBJECT_CLASS (ephy_download_parent_class)->dispose (object);
 }
 
-static void
-ephy_download_finalize (GObject *object)
-{
-  EphyDownload *download = EPHY_DOWNLOAD (object);
-  EphyDownloadPrivate *priv;
-
-  priv = download->priv;
-
-  g_free (priv->destination);
-  g_free (priv->source);
-
-  LOG ("EphyDownload finalised %p", object);
-
-  G_OBJECT_CLASS (ephy_download_parent_class)->finalize (object);
-}
-
 static void
 ephy_download_class_init (EphyDownloadClass *klass)
 {
@@ -688,7 +608,6 @@ ephy_download_class_init (EphyDownloadClass *klass)
   object_class->get_property = ephy_download_get_property;
   object_class->set_property = ephy_download_set_property;
   object_class->dispose = ephy_download_dispose;
-  object_class->finalize = ephy_download_finalize;
 
   /**
    * EphyDownload::download:
@@ -719,20 +638,6 @@ ephy_download_class_init (EphyDownloadClass *klass)
                                                         G_PARAM_STATIC_NAME |
                                                         G_PARAM_STATIC_NICK |
                                                         G_PARAM_STATIC_BLURB));
-  /**
-   * EphyDownload::source:
-   *
-   * Download's origin URI
-   */
-  g_object_class_install_property (object_class, PROP_SOURCE,
-                                   g_param_spec_string ("source",
-                                                        "Source",
-                                                        "Source URI",
-                                                        NULL,
-                                                        G_PARAM_READABLE |
-                                                        G_PARAM_STATIC_NAME |
-                                                        G_PARAM_STATIC_NICK |
-                                                        G_PARAM_STATIC_BLURB));
 
   /**
    * EphyDownload::action:
@@ -801,6 +706,22 @@ ephy_download_class_init (EphyDownloadClass *klass)
                                                         G_PARAM_STATIC_NICK |
                                                         G_PARAM_STATIC_BLURB));
 
+  /**
+   * EphyDownload::completed:
+   *
+   * The ::filename-suggested signal is emitted when we have received the
+   * suggested filename from WebKit.
+   **/
+  g_signal_new ("filename-suggested",
+                G_OBJECT_CLASS_TYPE (object_class),
+                G_SIGNAL_RUN_LAST,
+                G_STRUCT_OFFSET (EphyDownloadClass, filename_suggested),
+                NULL, NULL,
+                g_cclosure_marshal_generic,
+                G_TYPE_NONE,
+                1,
+                G_TYPE_STRING | G_SIGNAL_TYPE_STATIC_SCOPE);
+
   /**
    * EphyDownload::completed:
    *
@@ -837,11 +758,10 @@ ephy_download_init (EphyDownload *download)
   LOG ("EphyDownload initialising %p", download);
 
   download->priv->download = NULL;
-  download->priv->destination = NULL;
 
   download->priv->action = EPHY_DOWNLOAD_ACTION_NONE;
 
-  download->priv->start_time = 0;
+  download->priv->start_time = gtk_get_current_event_time ();
 
   download->priv->window = NULL;
   download->priv->widget = NULL;
@@ -852,33 +772,15 @@ download_decide_destination_cb (WebKitDownload *wk_download,
                                 const gchar *suggested_filename,
                                 EphyDownload *download)
 {
-  char *dest;
-
-  if (download->priv->destination) {
-    webkit_download_set_destination (wk_download, download->priv->destination);
+  if (webkit_download_get_destination (wk_download))
     return TRUE;
-  }
 
-  dest = define_destination_uri (download, suggested_filename);
-  if (!dest)
-    return FALSE;
+  g_signal_emit_by_name (download, "filename-suggested", suggested_filename);
 
-  ephy_download_set_destination_uri (download, dest);
-  webkit_download_set_destination (wk_download, dest);
-  g_free (dest);
+  if (webkit_download_get_destination (wk_download))
+    return TRUE;
 
-  return TRUE;
-}
-
-static void
-download_created_destination_cb (WebKitDownload *wk_download,
-                                 const gchar *destination,
-                                 EphyDownload *download)
-{
-  if (download->priv->destination)
-    return;
-
-  ephy_download_set_destination_uri (download, destination);
+  return set_destination_uri_for_suggested_filename (download, suggested_filename);
 }
 
 static void
@@ -896,8 +798,6 @@ download_finished_cb (WebKitDownload *wk_download,
     ephy_download_do_download_action (download, EPHY_DOWNLOAD_ACTION_AUTO);
   else
     ephy_download_do_download_action (download, priv->action);
-
-  ephy_embed_shell_remove_download (ephy_embed_shell_get_default (), download);
 }
 
 static void
@@ -915,22 +815,6 @@ download_failed_cb (WebKitDownload *wk_download,
 
 /**
  * ephy_download_new:
- * @parent: the #GtkWindow parent of the download, or %NULL
- *
- * Creates a new #EphyDownload. You can use ephy_download_new_for_download and
- * ephy_download_new_for_uri as convenience functions to create #EphyDownload
- * objects.
- *
- * Returns: an #EphyDownload.
- **/
-EphyDownload *
-ephy_download_new (GtkWindow *parent)
-{
-  return g_object_new (EPHY_TYPE_DOWNLOAD, "window", parent, NULL);
-}
-
-/**
- * ephy_download_new_for_download:
  * @download: a #WebKitDownload to wrap
  * @parent: the #GtkWindow parent of the download, or %NULL
  *
@@ -939,22 +823,18 @@ ephy_download_new (GtkWindow *parent)
  * Returns: an #EphyDownload.
  **/
 EphyDownload *
-ephy_download_new_for_download (WebKitDownload *download,
-                                GtkWindow *parent)
+ephy_download_new (WebKitDownload *download,
+                   GtkWindow *parent)
 {
   EphyDownload *ephy_download;
-  WebKitURIRequest *request;
 
   g_return_val_if_fail (WEBKIT_IS_DOWNLOAD (download), NULL);
 
-  ephy_download = ephy_download_new (parent);
+  ephy_download = g_object_new (EPHY_TYPE_DOWNLOAD, "window", parent, NULL);
 
   g_signal_connect (download, "decide-destination",
                     G_CALLBACK (download_decide_destination_cb),
                     ephy_download);
-  g_signal_connect (download, "created-destination",
-                    G_CALLBACK (download_created_destination_cb),
-                    ephy_download);
   g_signal_connect (download, "finished",
                     G_CALLBACK (download_finished_cb),
                     ephy_download);
@@ -964,11 +844,6 @@ ephy_download_new_for_download (WebKitDownload *download,
 
   ephy_download->priv->download = g_object_ref (download);
   g_object_set_data (G_OBJECT (download), "ephy-download-set", GINT_TO_POINTER (TRUE));
-  request = webkit_download_get_request (download);
-  ephy_download->priv->source = g_strdup (webkit_uri_request_get_uri (request));
-
-  /* In WebKit2 the download has already started */
-  ephy_embed_shell_add_download (ephy_embed_shell_get_default (), ephy_download);
 
   return ephy_download;
 }
@@ -993,7 +868,7 @@ ephy_download_new_for_uri (const char *uri,
 
   download = webkit_web_context_download_uri (webkit_web_context_get_default (), uri);
 
-  ephy_download = ephy_download_new_for_download (download, parent);
+  ephy_download = ephy_download_new (download, parent);
   g_object_unref (download);
 
   return ephy_download;
