@@ -28,8 +28,6 @@
 
 #include <gtk/gtk.h>
 
-#include "ephy-download.h"
-
 G_BEGIN_DECLS
 
 #define EPHY_TYPE_EMBED_SHELL   (ephy_embed_shell_get_type ())
@@ -46,6 +44,7 @@ typedef struct _EphyEmbedShellPrivate EphyEmbedShellPrivate;
 typedef enum
 {
   EPHY_EMBED_SHELL_MODE_BROWSER,
+  EPHY_EMBED_SHELL_MODE_STANDALONE,
   EPHY_EMBED_SHELL_MODE_PRIVATE,
   EPHY_EMBED_SHELL_MODE_INCOGNITO,
   EPHY_EMBED_SHELL_MODE_APPLICATION,
@@ -67,23 +66,15 @@ struct _EphyEmbedShellClass
 {
   GtkApplicationClass parent_class;
 
-  void    (* download_added)   (EphyEmbedShell *shell, EphyDownload *download);
-  void    (* download_removed) (EphyEmbedShell *shell, EphyDownload *download);
-
   void    (* prepare_close)    (EphyEmbedShell *shell);
 
   void    (* restored_window)  (EphyEmbedShell *shell);
-
-  /*< private >*/
-  GObject * (* get_embed_single)  (EphyEmbedShell *shell);
 };
 
 GType              ephy_embed_shell_get_type                   (void);
 EphyEmbedShell    *ephy_embed_shell_get_default                (void);
 GObject           *ephy_embed_shell_get_global_history_service (EphyEmbedShell   *shell);
 GObject           *ephy_embed_shell_get_encodings              (EphyEmbedShell   *shell);
-GObject           *ephy_embed_shell_get_embed_single           (EphyEmbedShell   *shell);
-GObject           *ephy_embed_shell_get_adblock_manager        (EphyEmbedShell   *shell);
 void               ephy_embed_shell_prepare_close              (EphyEmbedShell   *shell);
 void               ephy_embed_shell_restored_window            (EphyEmbedShell   *shell);
 void               ephy_embed_shell_set_page_setup             (EphyEmbedShell   *shell,
@@ -92,19 +83,13 @@ GtkPageSetup      *ephy_embed_shell_get_page_setup             (EphyEmbedShell
 void               ephy_embed_shell_set_print_settings         (EphyEmbedShell   *shell,
                                                                 GtkPrintSettings *settings);
 GtkPrintSettings  *ephy_embed_shell_get_print_settings         (EphyEmbedShell   *shell);
-GList             *ephy_embed_shell_get_downloads              (EphyEmbedShell   *shell);
-void               ephy_embed_shell_add_download               (EphyEmbedShell   *shell,
-                                                                EphyDownload     *download);
-void               ephy_embed_shell_remove_download            (EphyEmbedShell   *shell,
-                                                                EphyDownload     *download);
 EphyEmbedShellMode ephy_embed_shell_get_mode                   (EphyEmbedShell   *shell);
 gboolean           ephy_embed_shell_launch_handler             (EphyEmbedShell   *shell,
                                                                 GFile            *file,
                                                                 const char       *mime_type,
                                                                 guint32           user_time);
-#ifdef HAVE_WEBKIT2
+void               ephy_embed_shell_clear_cache                (EphyEmbedShell *shell);
 GDBusProxy        *ephy_embed_shell_get_web_extension_proxy    (EphyEmbedShell   *shell);
-#endif
 
 G_END_DECLS
 
