@@ -27,7 +27,6 @@
 #include "ephy-embed.h"
 
 #include "ephy-debug.h"
-#include "ephy-download.h"
 #include "ephy-embed-prefs.h"
 #include "ephy-embed-shell.h"
 #include "ephy-find-toolbar.h"
@@ -153,7 +152,7 @@ ephy_embed_set_statusbar_label (EphyEmbed *embed, const char *label)
 {
   EphyEmbedPrivate *priv = embed->priv;
 
-  nautilus_floating_bar_set_label (NAUTILUS_FLOATING_BAR (priv->floating_bar), label);
+  nautilus_floating_bar_set_primary_label (NAUTILUS_FLOATING_BAR (priv->floating_bar), label);
 
   if (label == NULL || label[0] == '\0') {
     gtk_widget_hide (priv->floating_bar);
@@ -481,16 +480,6 @@ ephy_embed_attach_inspector_cb (WebKitWebInspector *inspector,
   return TRUE;
 }
 
-void
-ephy_embed_auto_download_url (EphyEmbed *embed, const char *url)
-{
-  EphyDownload *download;
-
-  download = ephy_download_new_for_uri (url, NULL);
-  ephy_download_set_auto_destination (download);
-  ephy_download_set_action (download, EPHY_DOWNLOAD_ACTION_OPEN);
-}
-
 static void
 ephy_embed_set_fullscreen_message (EphyEmbed *embed,
                                    gboolean is_html5_fullscreen)
@@ -712,7 +701,7 @@ ephy_embed_constructed (GObject *object)
   ephy_embed_set_fullscreen_message (embed, FALSE);
 
   /* statusbar is hidden by default */
-  priv->floating_bar = nautilus_floating_bar_new (NULL, FALSE);
+  priv->floating_bar = nautilus_floating_bar_new (NULL, NULL, FALSE);
   gtk_widget_set_halign (priv->floating_bar, GTK_ALIGN_START);
   gtk_widget_set_valign (priv->floating_bar, GTK_ALIGN_END);
   gtk_widget_set_no_show_all (priv->floating_bar, TRUE);
