@@ -502,9 +502,8 @@ ephy_web_view_create_form_auth_save_confirmation_info_bar (EphyWebView *web_view
                                   GTK_ORIENTATION_HORIZONTAL);
 
   label = gtk_label_new (NULL);
-  /* Translators: The first %s is the username and the second one is the
-   * hostname where this is happening. Example: gnome@gmail.com and
-   * mail.google.com.
+  /* Translators: The %s the hostname where this is happening. 
+   * Example: mail.google.com.
    */
   message = g_markup_printf_escaped (_("Do you want to save your password for “%s”?"),
                                      hostname);
@@ -1330,7 +1329,7 @@ decide_policy_cb (WebKitWebView *web_view,
 
   /* If WebKit can't handle the mime type start the download
      process */
-  if (webkit_web_view_can_show_mime_type (web_view, mime_type))
+  if (webkit_response_policy_decision_is_mime_type_supported (response_decision))
     return FALSE;
 
   request = webkit_response_policy_decision_get_request (response_decision);
@@ -1742,6 +1741,8 @@ ephy_web_view_load_error_page (EphyWebView *view,
     reason = _("None specified");
 
   hostname = ephy_string_get_host_name (uri);
+  if (hostname == NULL)
+    hostname = g_strdup (uri);
 
   lang = g_strdup (pango_language_to_string (gtk_get_default_language ()));
   g_strdelimit (lang, "_-@", '\0');
@@ -1782,7 +1783,7 @@ ephy_web_view_load_error_page (EphyWebView *view,
       page_title = g_strdup_printf (_("Problem displaying “%s”"), hostname);
       msg_title = g_strdup (_("Oops!"));
       msg = g_strdup (_("Something went wrong while displaying this page. Please reload or visit a different page to continue."));
-      button_label = NULL;
+      button_label = g_strdup (_("Reload Anyway"));
 
       html_file = g_resources_lookup_data (EPHY_PAGE_TEMPLATE_PROCESS_CRASH, 0, NULL);
 
