@@ -319,7 +319,7 @@ ephy_file_helpers_init (const char *profile_dir,
 	private_profile = flags & EPHY_FILE_HELPERS_PRIVATE_PROFILE;
 	steal_data_from_profile = flags & EPHY_FILE_HELPERS_STEAL_DATA;
 
-	if (private_profile && profile_dir != NULL && !steal_data_from_profile)
+	if (profile_dir != NULL && !steal_data_from_profile)
 	{
 		dot_dir = g_strdup (profile_dir);
 	}
@@ -550,115 +550,6 @@ ephy_file_find (const char *path,
 	return ret;
 }
 
-/**
- * ephy_file_switch_temp_file:
- * @file_dest: destination file
- * @file_temp: file to move to @file
- *
- * Moves @file_temp to @file_dest atomically, doing a backup and restoring it if
- * something fails.
- *
- * Returns: %TRUE if the switch was successful
- **/
-gboolean
-ephy_file_switch_temp_file (GFile *file_dest,
-			    GFile *file_temp)
-{
-	char *file_dest_path, *file_temp_path;
-	char *backup_path;
-	gboolean dest_exists;
-	gboolean retval = TRUE;
-	GFile *backup_file;
-	GError *error = NULL;
-
-	file_dest_path = g_file_get_path (file_dest);
-	file_temp_path = g_file_get_path (file_temp);
-
-	dest_exists = g_file_test (file_dest_path, G_FILE_TEST_EXISTS);
-
-	backup_path = g_strconcat (file_dest_path, ".old", NULL);
-	backup_file = g_file_new_for_path (backup_path);
-
-	if (dest_exists)
-	{
-		if (g_file_move (file_dest, backup_file,
-				 G_FILE_COPY_OVERWRITE,
-				 NULL, NULL, NULL, &error) == FALSE)
-		{
-			g_warning ("Failed to backup %s to %s: %s",
-				   file_dest_path, backup_path,
-				   error ? error->message : "No error set");
-
-			if (error)
-			{
-				g_error_free (error);
-				error = NULL;
-			}
-
-			retval = FALSE;
-			goto failed;
-		}
-	}
-
-	if (g_file_move (file_temp, file_dest,
-			 G_FILE_COPY_OVERWRITE,
-			 NULL, NULL, NULL, &error) == FALSE)
-	{
-		g_warning ("Failed to replace %s with %s: %s",
-			   file_temp_path, file_dest_path,
-			   error ? error->message : "No error set");
-
-		if (error)
-		{
-			g_error_free (error);
-			error = NULL;
-		}
-
-		if (g_file_move (backup_file, file_dest,
-				 G_FILE_COPY_OVERWRITE,
-				 NULL, NULL, NULL, &error) == FALSE)
-		{
-			g_warning ("Failed to restore %s from %s: %s",
-				   file_dest_path, file_temp_path,
-				   error ? error->message : "No error set");
-
-			if (error)
-			{
-				g_error_free (error);
-				error = NULL;
-			}
-		}
-
-		retval = FALSE;
-		goto failed;
-	}
-
-	if (dest_exists)
-	{
-		if (g_file_delete (backup_file, NULL, &error) == FALSE)
-		{
-			g_warning ("Failed to delete old file %s: %s",
-				   backup_path,
-				   error ? error->message : "No error set");
-
-			if (error)
-			{
-				g_error_free (error);
-				error = NULL;
-			}
-		}
-	}
-
-failed:
-	g_free (file_dest_path);
-	g_free (file_temp_path);
-
-	g_free (backup_path);
-	g_object_unref (backup_file);
-
-	return retval;
-}
-
 /**
  * ephy_file_delete_on_exit:
  * @file: a #GFile
@@ -1035,6 +926,41 @@ ephy_file_delete_uri (const char *uri)
 	g_object_unref (file);
 }
 
+/**
+ * ephy_file_move_uri
+ * @source_uri: URI of the file to be moved
+ * @dest_uri: URI of the destination
+ *
+ * Move from source_uri to dest_uri, overwriting if necessary.
+ */
+gboolean
+ephy_file_move_uri (const char *source_uri, const char *dest_uri)
+{
+	GFile *src;
+	GFile *dest;
+	gboolean ret;
+
+	g_return_if_fail (source_uri && dest_uri);
+
+	src = g_file_new_for_uri (source_uri);
+	dest = g_file_new_for_uri (dest_uri);
+
+	ret = g_file_move (src, dest, G_FILE_COPY_OVERWRITE | G_FILE_COPY_ALL_METADATA,
+			   NULL, NULL, NULL, NULL);
+
+	if (ret == TRUE)
+	{
+		LOG ("Moved file '%s' to '%s'", source_uri, dest_uri);
+	}
+	else
+	{
+		LOG ("Couldn't move file '%s' to '%s'", source_uri, dest_uri);
+	}
+	g_object_unref (src);
+	g_object_unref (dest);
+	return ret;
+}
+
 /**
  * ephy_file_create_data_uri_for_filename:
  * @filename: the filename of a local path
