@@ -80,6 +80,7 @@ static void ephy_history_service_quit                                     (EphyH
 enum {
   PROP_0,
   PROP_HISTORY_FILENAME,
+  PROP_READ_ONLY
 };
 
 #define EPHY_HISTORY_SERVICE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE((o), EPHY_TYPE_HISTORY_SERVICE, EphyHistoryServicePrivate))
@@ -96,6 +97,9 @@ ephy_history_service_set_property (GObject *object, guint property_id, const GVa
       g_free (self->priv->history_filename);
       self->priv->history_filename = g_strdup (g_value_get_string (value));
       break;
+    case PROP_READ_ONLY:
+      self->priv->read_only = g_value_get_boolean (value);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (self, property_id, pspec);
       break;
@@ -268,6 +272,13 @@ ephy_history_service_class_init (EphyHistoryServiceClass *klass)
                                                         "The filename of the SQLite file holding containing history",
                                                         NULL,
                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+    g_object_class_install_property (gobject_class,
+                                     PROP_READ_ONLY,
+                                     g_param_spec_boolean ("read-only",
+                                                           "Read only mode",
+                                                           "Whether the history service works in read only mode",
+                                                           FALSE,
+                                                           G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
   g_type_class_add_private (gobject_class, sizeof (EphyHistoryServicePrivate));
 }
@@ -282,10 +293,12 @@ ephy_history_service_init (EphyHistoryService *self)
 }
 
 EphyHistoryService *
-ephy_history_service_new (const char *history_filename)
+ephy_history_service_new (const char *history_filename,
+                          gboolean read_only)
 {
   return EPHY_HISTORY_SERVICE (g_object_new (EPHY_TYPE_HISTORY_SERVICE,
                                              "history-filename", history_filename,
+                                             "read-only", read_only,
                                               NULL));
 }
 
@@ -295,6 +308,40 @@ sort_messages (EphyHistoryServiceMessage* a, EphyHistoryServiceMessage* b, gpoin
   return a->type > b->type ? 1 : a->type == b->type ? 0 : -1;
 }
 
+static EphyHistoryServiceMessage *
+ephy_history_service_message_new (EphyHistoryService *service,
+                                  EphyHistoryServiceMessageType type,
+                                  gpointer method_argument,
+                                  GDestroyNotify method_argument_cleanup,
+                                  GCancellable *cancellable,
+                                  EphyHistoryJobCallback callback,
+                                  gpointer user_data)
+{
+  EphyHistoryServiceMessage *message = g_slice_alloc0 (sizeof (EphyHistoryServiceMessage));
+
+  message->service = service;
+  message->type = type;
+  message->method_argument = method_argument;
+  message->method_argument_cleanup = method_argument_cleanup;
+  message->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
+  message->callback = callback;
+  message->user_data = user_data;
+
+  return message;
+}
+
+static void
+ephy_history_service_message_free (EphyHistoryServiceMessage *message)
+{
+  if (message->method_argument_cleanup)
+    message->method_argument_cleanup (message->method_argument);
+
+  if (message->cancellable)
+    g_object_unref (message->cancellable);
+
+  g_slice_free1 (sizeof (EphyHistoryServiceMessage), message);
+}
+
 static void
 ephy_history_service_send_message (EphyHistoryService *self, EphyHistoryServiceMessage *message)
 {
@@ -313,6 +360,9 @@ ephy_history_service_commit (EphyHistoryService *self)
   if (NULL == priv->history_database)
     return;
 
+  if (priv->read_only)
+    return;
+
   ephy_sqlite_connection_commit_transaction (priv->history_database, &error);
   if (NULL != error) {
     g_error ("Could not commit idle history database transaction: %s", error->message);
@@ -358,7 +408,7 @@ ephy_history_service_open_database_connections (EphyHistoryService *self)
   if (error) {
     g_object_unref (priv->history_database);
     priv->history_database = NULL;
-    g_error ("Could not open history database at %s: %s", priv->history_filename, error->message);
+    g_warning ("Could not open history database at %s: %s", priv->history_filename, error->message);
     g_error_free (error);
     return FALSE;
   }
@@ -372,9 +422,10 @@ ephy_history_service_open_database_connections (EphyHistoryService *self)
     return FALSE;
   }
 
-  if ((ephy_history_service_initialize_hosts_table (self) == FALSE) ||
-      (ephy_history_service_initialize_urls_table (self) == FALSE) ||
-      (ephy_history_service_initialize_visits_table (self) == FALSE))
+  if (!priv->read_only &&
+      ((ephy_history_service_initialize_hosts_table (self) == FALSE) ||
+       (ephy_history_service_initialize_urls_table (self) == FALSE) ||
+       (ephy_history_service_initialize_visits_table (self) == FALSE)))
     return FALSE;
 
   return TRUE;
@@ -401,6 +452,9 @@ ephy_history_service_clear_all (EphyHistoryService *self)
   if (NULL == priv->history_database)
     return;
 
+  if (priv->read_only)
+    return;
+
   ephy_sqlite_connection_execute (priv->history_database,
                                   "DELETE FROM hosts;", &error);
   if (error) {
@@ -424,7 +478,8 @@ ephy_history_service_is_scheduled_to_commit (EphyHistoryService *self)
 void
 ephy_history_service_schedule_commit (EphyHistoryService *self)
 {
-  self->priv->scheduled_to_commit = TRUE;
+  if (!self->priv->read_only)
+    self->priv->scheduled_to_commit = TRUE;
 }
 
 static gboolean
@@ -475,40 +530,6 @@ run_history_service_thread (EphyHistoryService *self)
   return NULL;
 }
 
-static EphyHistoryServiceMessage *
-ephy_history_service_message_new (EphyHistoryService *service,
-                                  EphyHistoryServiceMessageType type,
-                                  gpointer method_argument,
-                                  GDestroyNotify method_argument_cleanup,
-                                  GCancellable *cancellable,
-                                  EphyHistoryJobCallback callback,
-                                  gpointer user_data)
-{
-  EphyHistoryServiceMessage *message = g_slice_alloc0 (sizeof (EphyHistoryServiceMessage));
-
-  message->service = service; 
-  message->type = type;
-  message->method_argument = method_argument;
-  message->method_argument_cleanup = method_argument_cleanup;
-  message->cancellable = cancellable ? g_object_ref (cancellable) : NULL;
-  message->callback = callback;
-  message->user_data = user_data;
-
-  return message;
-}
-
-static void
-ephy_history_service_message_free (EphyHistoryServiceMessage *message)
-{
-  if (message->method_argument_cleanup)
-    message->method_argument_cleanup (message->method_argument);
-
-  if (message->cancellable)
-    g_object_unref (message->cancellable);
-
-  g_slice_free1 (sizeof (EphyHistoryServiceMessage), message);
-}
-
 static gboolean
 ephy_history_service_execute_job_callback (gpointer data)
 {
@@ -610,6 +631,9 @@ ephy_history_service_execute_add_visit (EphyHistoryService *self, EphyHistoryPag
   gboolean success;
   g_assert (self->priv->history_thread == g_thread_self ());
 
+  if (self->priv->read_only)
+    return FALSE;
+
   success = ephy_history_service_execute_add_visit_helper (self, visit);
   return success;
 }
@@ -620,6 +644,9 @@ ephy_history_service_execute_add_visits (EphyHistoryService *self, GList *visits
   gboolean success = TRUE;
   g_assert (self->priv->history_thread == g_thread_self ());
 
+  if (self->priv->read_only)
+    return FALSE;
+
   while (visits) {
     success = success && ephy_history_service_execute_add_visit_helper (self, (EphyHistoryPageVisit *) visits->data);
     visits = visits->next;
@@ -809,6 +836,9 @@ ephy_history_service_execute_set_url_title (EphyHistoryService *self,
 {
   char *title = g_strdup (url->title);
 
+  if (self->priv->read_only)
+    return FALSE;
+
   if (NULL == ephy_history_service_get_url_row (self, NULL, url)) {
     /* The URL is not yet in the database, so we can't update it.. */
     g_free (title);
@@ -861,6 +891,9 @@ ephy_history_service_execute_set_url_zoom_level (EphyHistoryService *self,
   double zoom_level;
   EphyHistoryHost *host;
 
+  if (self->priv->read_only)
+    return FALSE;
+
   g_variant_get (variant, "(sd)", &url_string, &zoom_level);
 
   host = ephy_history_service_get_host_row_from_url (self, url_string);
@@ -904,6 +937,9 @@ ephy_history_service_execute_set_url_hidden (EphyHistoryService *self,
 {
   gboolean hidden;
 
+  if (self->priv->read_only)
+    return FALSE;
+
   hidden = url->hidden;
 
   if (NULL == ephy_history_service_get_url_row (self, NULL, url)) {
@@ -947,6 +983,9 @@ ephy_history_service_execute_set_url_thumbnail_time (EphyHistoryService *self,
 {
   int thumbnail_time;
 
+  if (self->priv->read_only)
+    return FALSE;
+
   thumbnail_time = url->thumbnail_time;
 
   if (NULL == ephy_history_service_get_url_row (self, NULL, url))
@@ -1065,6 +1104,9 @@ ephy_history_service_execute_delete_urls (EphyHistoryService *self,
   EphyHistoryURL *url;
   SignalEmissionContext *ctx;
 
+  if (self->priv->read_only)
+    return FALSE;
+
   for (l = urls; l != NULL; l = l->next) {
     url = l->data;
     ephy_history_service_delete_url (self, url);
@@ -1101,6 +1143,9 @@ ephy_history_service_execute_delete_host (EphyHistoryService *self,
 {
   SignalEmissionContext *ctx;
 
+  if (self->priv->read_only)
+    return FALSE;
+
   ephy_history_service_delete_host_row (self, host);
   ephy_history_service_schedule_commit (self);
 
@@ -1119,6 +1164,8 @@ ephy_history_service_execute_clear (EphyHistoryService *self,
                                     gpointer pointer,
                                     gpointer *result)
 {
+  if (self->priv->read_only)
+    return FALSE;
 
   ephy_history_service_clear_all (self);
   ephy_history_service_schedule_commit (self);
@@ -1227,7 +1274,10 @@ ephy_history_service_process_message (EphyHistoryService *self,
 
   method = methods[message->type];
   message->result = NULL;
-  message->success = method (message->service, message->method_argument, &message->result);
+  if (message->service->priv->history_database)
+    message->success = method (message->service, message->method_argument, &message->result);
+  else
+    message->success = FALSE;
 
   if (message->callback || message->type == CLEAR)
     g_idle_add ((GSourceFunc)ephy_history_service_execute_job_callback, message);
