@@ -45,7 +45,7 @@ struct _EphyDownloadWidgetPrivate
   GtkWidget *text;
   GtkWidget *remaining;
   GtkWidget *button;
-  GtkWidget *menu;
+  GtkWidget *menu_button;
   GtkWidget *icon;
 
   gboolean finished;
@@ -95,28 +95,45 @@ get_destination_basename_from_download (EphyDownload *ephy_download)
   return unescaped;
 }
 
-static char *
-format_interval (gdouble interval)
+/* modified from telepathy-account-widgets/tpaw-time.c */
+static gchar *
+duration_to_string (guint seconds)
 {
-   int hours, mins, secs;
-
-   hours = (int) (interval / 3600);
-   interval -= hours * 3600;
-   mins = (int) (interval / 60);
-   interval -= mins * 60;
-   secs = (int) interval;
-
-   if (hours > 0) {
-     if (mins > 0)
-       return g_strdup_printf (ngettext ("%u:%02u hour left", "%u:%02u hours left", hours), hours, mins);
-     else
-       return g_strdup_printf (ngettext ("%u hour left", "%u hours left", hours), hours);
-   } else {
-     if (mins > 0)
-       return g_strdup_printf (ngettext ("%u:%02u minute left", "%u:%02u minutes left", mins), mins, secs);
-     else
-       return g_strdup_printf (ngettext ("%u second left", "%u seconds left", secs), secs);
-   }
+  if (seconds < 60)
+    {
+      return g_strdup_printf (ngettext ("%d second left",
+        "%d seconds left", seconds), seconds);
+    }
+  else if (seconds < (60 * 60))
+    {
+      seconds /= 60;
+      return g_strdup_printf (ngettext ("%d minute left",
+        "%d minutes left", seconds), seconds);
+    }
+  else if (seconds < (60 * 60 * 24))
+    {
+      seconds /= 60 * 60;
+      return g_strdup_printf (ngettext ("%d hour left",
+        "%d hours left", seconds), seconds);
+    }
+  else if (seconds < (60 * 60 * 24 * 7))
+    {
+      seconds /= 60 * 60 * 24;
+      return g_strdup_printf (ngettext ("%d day left",
+        "%d days left", seconds), seconds);
+    }
+  else if (seconds < (60 * 60 * 24 * 30))
+    {
+      seconds /= 60 * 60 * 24 * 7;
+      return g_strdup_printf (ngettext ("%d week left",
+        "%d weeks left", seconds), seconds);
+    }
+  else
+    {
+      seconds /= 60 * 60 * 24 * 30;
+      return g_strdup_printf (ngettext ("%d month left",
+        "%d months left", seconds), seconds);
+    }
 }
 
 static gdouble
@@ -170,6 +187,16 @@ update_download_icon (EphyDownloadWidget *widget)
   g_object_unref (new_icon);
 }
 
+static void
+update_download_destination (EphyDownloadWidget *widget)
+{
+  char *dest;
+
+  dest = get_destination_basename_from_download (widget->priv->download);
+  gtk_label_set_text (GTK_LABEL (widget->priv->text), dest);
+  g_free (dest);
+}
+
 static void
 update_download_label_and_tooltip (EphyDownloadWidget *widget,
                                    const char *download_label)
@@ -177,9 +204,14 @@ update_download_label_and_tooltip (EphyDownloadWidget *widget,
   WebKitDownload *download;
   char *remaining_tooltip;
   char *destination;
+  const char *dest;
 
   download = ephy_download_get_webkit_download (widget->priv->download);
-  destination = g_filename_display_basename (webkit_download_get_destination (download));
+  dest = webkit_download_get_destination (download);
+  if (dest == NULL)
+    return;
+
+  destination = g_filename_display_basename (dest);
 
   remaining_tooltip = g_markup_printf_escaped ("%s\n%s", destination, download_label);
   g_free (destination);
@@ -221,7 +253,7 @@ widget_progress_cb (WebKitDownload *download,
     if (time > 0) {
       char *remaining;
 
-      remaining = format_interval (time);
+      remaining = duration_to_string ((guint)time);
       download_label = g_strdup_printf ("%d%% (%s)", progress, remaining);
       g_free (remaining);
     }
@@ -240,18 +272,6 @@ widget_progress_cb (WebKitDownload *download,
   }
 }
 
-static void
-widget_destination_changed_cb (WebKitDownload *download,
-                               GParamSpec *pspec,
-                               EphyDownloadWidget *widget)
-{
-  char *dest;
-
-  dest = get_destination_basename_from_download (widget->priv->download);
-  gtk_label_set_text (GTK_LABEL (widget->priv->text), dest);
-  g_free (dest);
-}
-
 static void
 widget_finished_cb (WebKitDownload *download,
                     EphyDownloadWidget *widget)
@@ -268,12 +288,16 @@ widget_failed_cb (WebKitDownload *download,
 {
   char *error_msg;
 
+  g_signal_handlers_disconnect_by_func (download, widget_finished_cb, widget);
   g_signal_handlers_disconnect_by_func (download, widget_progress_cb, widget);
 
+  widget->priv->finished = TRUE;
   error_msg = g_strdup_printf (_("Error downloading: %s"), error->message);
   gtk_label_set_text (GTK_LABEL (widget->priv->remaining), error_msg);
   gtk_widget_set_tooltip_text (GTK_WIDGET (widget), error_msg);
   g_free (error_msg);
+
+  totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (widget->priv->button), TRUE);
 }
 
 static void
@@ -298,28 +322,24 @@ cancel_activate_cb (GtkMenuItem *item, EphyDownloadWidget *widget)
 }
 
 static void
-download_menu_clicked_cb (GtkWidget *button,
-                          GdkEventButton *event,
-                          EphyDownloadWidget *widget)
+add_popup_menu (EphyDownloadWidget *widget)
 {
   GtkWidget *item;
   GtkWidget *menu;
-  GtkWidget *box;
-  GList *children = NULL;
   char *basename, *name;
   WebKitDownload *download;
+  const char *dest;
 
   download = ephy_download_get_webkit_download (widget->priv->download);
+  dest = webkit_download_get_destination (download);
+  if (dest == NULL)
+    return;
 
-  basename = g_filename_display_basename (webkit_download_get_destination (download));
+  basename = g_filename_display_basename (dest);
   name = g_uri_unescape_string (basename, NULL);
 
-  box = gtk_widget_get_parent (button);
-  children = gtk_container_get_children (GTK_CONTAINER (box));
-  totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (children->data), FALSE);
-  g_list_free (children);
-
   menu = gtk_menu_new ();
+  gtk_widget_set_halign (menu, GTK_ALIGN_END);
 
   item = gtk_menu_item_new_with_label (name);
   gtk_widget_set_sensitive (item, FALSE);
@@ -350,9 +370,84 @@ download_menu_clicked_cb (GtkWidget *button,
 
   gtk_widget_show_all (menu);
 
-  gtk_menu_attach_to_widget (GTK_MENU (menu), button, NULL);
-  gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
-                  event->button, event->time);
+  gtk_menu_button_set_popup (GTK_MENU_BUTTON (widget->priv->menu_button), menu);
+}
+
+static void
+widget_destination_changed_cb (WebKitDownload *download,
+                               GParamSpec *pspec,
+                               EphyDownloadWidget *widget)
+{
+  update_download_destination (widget);
+  add_popup_menu (widget);
+}
+
+static void
+stop_glowing (EphyDownloadWidget *widget)
+{
+  totem_glow_button_set_glow (TOTEM_GLOW_BUTTON (widget->priv->button), FALSE);
+}
+
+static void
+disconnect_download (EphyDownloadWidget *widget)
+{
+  WebKitDownload *download;
+
+  download = ephy_download_get_webkit_download (widget->priv->download);
+
+  g_signal_handlers_disconnect_by_func (download, widget_progress_cb, widget);
+  g_signal_handlers_disconnect_by_func (download, widget_destination_changed_cb, widget);
+  g_signal_handlers_disconnect_by_func (download, widget_finished_cb, widget);
+  g_signal_handlers_disconnect_by_func (download, widget_failed_cb, widget);
+
+  ephy_download_set_widget (widget->priv->download, NULL);
+}
+
+static void
+connect_download (EphyDownloadWidget *widget)
+{
+  WebKitDownload *download;
+
+  download = ephy_download_get_webkit_download (widget->priv->download);
+
+  g_signal_connect (download, "notify::estimated-progress",
+                    G_CALLBACK (widget_progress_cb), widget);
+  g_signal_connect (download, "notify::destination",
+                    G_CALLBACK (widget_destination_changed_cb), widget);
+  g_signal_connect (download, "finished",
+                    G_CALLBACK (widget_finished_cb), widget);
+  g_signal_connect (download, "failed",
+                    G_CALLBACK (widget_failed_cb), widget);
+
+  ephy_download_set_widget (widget->priv->download, GTK_WIDGET (widget));
+}
+
+static void
+ephy_download_widget_set_download (EphyDownloadWidget *widget,
+                                   EphyDownload       *download)
+{
+  g_return_val_if_fail (EPHY_IS_DOWNLOAD_WIDGET (widget), NULL);
+
+  if (widget->priv->download == download)
+    return;
+
+  if (widget->priv->download != NULL) {
+    disconnect_download (widget);
+    g_object_unref (widget->priv->download);
+  }
+
+  widget->priv->download = NULL;
+
+  if (download != NULL) {
+    widget->priv->download = g_object_ref (download);
+    connect_download (widget);
+  }
+
+  update_download_icon (widget);
+  update_download_destination (widget);
+  add_popup_menu (widget);
+
+  g_object_notify (G_OBJECT (widget), "download");
 }
 
 static void
@@ -386,7 +481,7 @@ ephy_download_widget_set_property (GObject      *object,
 
   switch (property_id) {
     case PROP_DOWNLOAD:
-      widget->priv->download = g_object_ref (g_value_get_object (value));
+      ephy_download_widget_set_download (widget, g_value_get_object (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -398,20 +493,13 @@ static void
 ephy_download_widget_dispose (GObject *object)
 {
   EphyDownloadWidget *widget;
-  WebKitDownload *download;
 
   LOG ("EphyDownloadWidget %p dispose", object);
 
   widget = EPHY_DOWNLOAD_WIDGET (object);
 
   if (widget->priv->download != NULL) {
-    download = ephy_download_get_webkit_download (widget->priv->download);
-
-    g_signal_handlers_disconnect_by_func (download, widget_progress_cb, widget);
-    g_signal_handlers_disconnect_by_func (download, widget_destination_changed_cb, widget);
-    g_signal_handlers_disconnect_by_func (download, widget_finished_cb, widget);
-    g_signal_handlers_disconnect_by_func (download, widget_failed_cb, widget);
-
+    disconnect_download (widget);
     g_object_unref (widget->priv->download);
     widget->priv->download = NULL;
   }
@@ -447,6 +535,73 @@ ephy_download_widget_class_init (EphyDownloadWidgetClass *klass)
                                                         G_PARAM_STATIC_BLURB));
 }
 
+static void
+smallify_label (GtkLabel *label)
+{
+        PangoAttrList *attrs;
+        attrs = pango_attr_list_new ();
+        pango_attr_list_insert (attrs, pango_attr_scale_new (PANGO_SCALE_SMALL));
+        gtk_label_set_attributes (label, attrs);
+        pango_attr_list_unref (attrs);
+}
+
+static void
+create_widget (EphyDownloadWidget *widget)
+{
+
+  GtkWidget *grid;
+  GtkWidget *icon;
+  GtkWidget *text;
+  GtkWidget *button;
+  GtkWidget *menu_button;
+  GtkWidget *remain;
+
+  grid = gtk_grid_new ();
+  gtk_grid_set_column_spacing (GTK_GRID (grid), 6);
+
+  button = totem_glow_button_new ();
+  menu_button = gtk_menu_button_new ();
+  gtk_menu_button_set_direction (GTK_MENU_BUTTON (menu_button), GTK_ARROW_UP);
+
+  icon = gtk_image_new ();
+
+  text = gtk_label_new (NULL);
+  smallify_label (GTK_LABEL (text));
+  gtk_misc_set_alignment (GTK_MISC (text), 0, 0.5);
+  gtk_label_set_ellipsize (GTK_LABEL (text), PANGO_ELLIPSIZE_END);
+
+  remain = gtk_label_new (_("Starting…"));
+  smallify_label (GTK_LABEL (remain));
+  gtk_misc_set_alignment (GTK_MISC (remain), 0, 0.5);
+  gtk_label_set_ellipsize (GTK_LABEL (remain), PANGO_ELLIPSIZE_END);
+
+  gtk_grid_attach (GTK_GRID (grid), icon, 0, 0, 1, 2);
+  gtk_grid_attach (GTK_GRID (grid), text, 1, 0, 1, 1);
+  gtk_grid_attach (GTK_GRID (grid), remain, 1, 1, 1, 1);
+
+  widget->priv->text = text;
+  widget->priv->icon = icon;
+  widget->priv->button = button;
+  widget->priv->remaining = remain;
+  widget->priv->menu_button = menu_button;
+
+  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_HALF);
+
+  gtk_container_add (GTK_CONTAINER (button), grid);
+
+  gtk_box_pack_start (GTK_BOX (widget), button, FALSE, FALSE, 0);
+  gtk_box_pack_end (GTK_BOX (widget), menu_button, FALSE, FALSE, 0);
+
+  g_signal_connect (button, "clicked",
+                    G_CALLBACK (download_clicked_cb), widget);
+  g_signal_connect_swapped (menu_button, "clicked",
+                            G_CALLBACK (stop_glowing), widget);
+
+  gtk_widget_show_all (button);
+  gtk_widget_show_all (menu_button);
+
+}
+
 static void
 ephy_download_widget_init (EphyDownloadWidget *self)
 {
@@ -454,6 +609,8 @@ ephy_download_widget_init (EphyDownloadWidget *self)
 
   self->priv = DOWNLOAD_WIDGET_PRIVATE (self);
 
+  create_widget (self);
+
   gtk_orientable_set_orientation (GTK_ORIENTABLE (self),
                                   GTK_ORIENTATION_HORIZONTAL);
   context = gtk_widget_get_style_context (GTK_WIDGET (self));
@@ -504,82 +661,10 @@ ephy_download_widget_new (EphyDownload *ephy_download)
 {
   EphyDownloadWidget *widget;
 
-  GtkWidget *grid;
-  GtkWidget *icon;
-  GtkWidget *text;
-  GtkWidget *button;
-  GtkWidget *menu;
-  GtkWidget *remain;
-
-  char *dest;
-  WebKitDownload *download;
-  GIcon *gicon;
-
   g_return_val_if_fail (EPHY_IS_DOWNLOAD (ephy_download), NULL);
 
   widget = g_object_new (EPHY_TYPE_DOWNLOAD_WIDGET,
                          "download", ephy_download, NULL);
-  download = ephy_download_get_webkit_download (ephy_download);
-
-  grid = gtk_grid_new ();
-
-  button = totem_glow_button_new ();
-  menu = gtk_button_new ();
-
-  gicon = get_gicon_from_download (ephy_download);
-  icon = gtk_image_new_from_gicon (gicon, GTK_ICON_SIZE_LARGE_TOOLBAR);
-  g_object_unref (gicon);
-
-  dest = get_destination_basename_from_download (ephy_download);
-  text = gtk_label_new (dest);
-  gtk_misc_set_alignment (GTK_MISC (text), 0, 0.5);
-  gtk_label_set_ellipsize (GTK_LABEL (text), PANGO_ELLIPSIZE_END);
-
-  remain = gtk_label_new (_("Starting…"));
-  gtk_misc_set_alignment (GTK_MISC (remain), 0, 0.5);
-  gtk_label_set_ellipsize (GTK_LABEL (remain), PANGO_ELLIPSIZE_END);
-
-  gtk_grid_attach (GTK_GRID (grid), icon, 0, 0, 1, 2);
-  gtk_grid_attach (GTK_GRID (grid), text, 1, 0, 1, 1);
-  gtk_grid_attach (GTK_GRID (grid), remain, 1, 1, 1, 1);
-
-  gtk_widget_set_tooltip_text (GTK_WIDGET (widget), dest);
-  g_free (dest);
-
-  widget->priv->text = text;
-  widget->priv->icon = icon;
-  widget->priv->button = button;
-  widget->priv->remaining = remain;
-  widget->priv->menu = menu;
-
-  g_signal_connect (download, "notify::estimated-progress",
-                    G_CALLBACK (widget_progress_cb), widget);
-  g_signal_connect (download, "notify::destination",
-                    G_CALLBACK (widget_destination_changed_cb), widget);
-  g_signal_connect (download, "finished",
-                    G_CALLBACK (widget_finished_cb), widget);
-  g_signal_connect (download, "failed",
-                    G_CALLBACK (widget_failed_cb), widget);
-
-  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_HALF);
-  gtk_button_set_relief (GTK_BUTTON (menu), GTK_RELIEF_NORMAL);
-
-  gtk_container_add (GTK_CONTAINER (button), grid);
-  gtk_container_add (GTK_CONTAINER (menu),
-                     gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_NONE));
-
-  gtk_box_pack_start (GTK_BOX (widget), button, FALSE, FALSE, 0);
-  gtk_box_pack_end (GTK_BOX (widget), menu, FALSE, FALSE, 0);
-
-  g_signal_connect (button, "clicked",
-                    G_CALLBACK (download_clicked_cb), widget);
-  g_signal_connect (menu, "button-press-event",
-                    G_CALLBACK (download_menu_clicked_cb), widget);
-
-  gtk_widget_show_all (button);
-  gtk_widget_show_all (menu);
-
-  ephy_download_set_widget (ephy_download, GTK_WIDGET (widget));
 
   return GTK_WIDGET (widget);
 }
