@@ -24,6 +24,7 @@
 #include "config.h"
 #include "ephy-location-entry.h"
 
+#include "ephy-widgets-type-builtins.h"
 #include "ephy-about-handler.h"
 #include "ephy-debug.h"
 #include "ephy-dnd.h"
@@ -109,7 +110,7 @@ enum
 	PROP_0,
 	PROP_LOCATION,
 	PROP_FAVICON,
-	PROP_LOCK_STOCK,
+	PROP_LOCK_STATE,
 	PROP_SHOW_LOCK,
 	PROP_SHOW_FAVICON
 };
@@ -144,9 +145,9 @@ ephy_location_entry_set_property (GObject *object,
 		ephy_location_entry_set_favicon (entry,
 						 g_value_get_object (value));
 		break;
-	case PROP_LOCK_STOCK:
-		ephy_location_entry_set_lock_stock (entry,
-						    g_value_get_string (value));
+	case PROP_LOCK_STATE:
+		ephy_location_entry_set_lock_state (entry,
+						    g_value_get_enum (value));
 		break;
 	case PROP_SHOW_LOCK:
 		ephy_location_entry_set_show_lock (entry,
@@ -255,17 +256,18 @@ ephy_location_entry_class_init (EphyLocationEntryClass *klass)
 							      G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	/**
-	* EphyLocationEntry:lock-stock-id:
+	* EphyLocationEntry:lock-state:
 	*
-	* Stock id of the security icon.
+	* State of the security icon.
 	*/
 	g_object_class_install_property (object_class,
-					 PROP_LOCK_STOCK,
-					 g_param_spec_string  ("lock-stock-id",
-							       "Lock Stock ID",
-							       "Stock id of the security icon",
-							       NULL,
-							       G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
+					 PROP_LOCK_STATE,
+					 g_param_spec_enum  ("lock-state",
+							     "Lock state",
+							     "State of the security icon",
+							     EPHY_TYPE_LOCATION_LOCK_STATE,
+							     EPHY_LOCATION_LOCK_STATE_UNKNOWN,
+							     G_PARAM_WRITABLE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB));
 
 	/**
 	* EphyLocationEntry:show-lock:
@@ -669,8 +671,21 @@ each_url_get_data_binder (EphyDragEachSelectedItemDataGet iteratee,
 	g_free (title);
 }
 
-#define DRAG_ICON_LAYOUT_PADDING	2
-#define DRAG_ICON_ICON_PADDING		4
+static void
+sanitize_location (char **url)
+{
+	char *str;
+
+	/* Do not show internal ephy-about: protocol to users */
+	if (g_str_has_prefix (*url, EPHY_ABOUT_SCHEME)) {
+		str = g_strdup_printf ("about:%s", *url + strlen (EPHY_ABOUT_SCHEME) + 1);
+		g_free (*url);
+		*url = str;
+	}
+}
+
+#define DRAG_ICON_LAYOUT_PADDING	5
+#define DRAG_ICON_ICON_PADDING		10
 #define DRAG_ICON_MAX_WIDTH_CHARS	32
 
 static cairo_surface_t *
@@ -693,10 +708,12 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 	cairo_t *cr;
 	GtkStateFlags state;
 	GdkRGBA color;
+	GdkPixbuf *favicon;
 
 	state = gtk_widget_get_state_flags (widget);
 
 	g_signal_emit (entry, signals[GET_LOCATION], 0, &address);
+	sanitize_location (&address);
 	g_signal_emit (entry, signals[GET_TITLE], 0, &title);
 	if (address == NULL || title == NULL) return NULL;
 
@@ -716,9 +733,16 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 	}
 
 	if (priv->favicon != NULL)
+		favicon = g_object_ref (priv->favicon);
+	else
+		favicon = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (),
+						    "text-x-generic-symbolic",
+						    16,
+						    0, NULL);
+	if (favicon != NULL)
 	{
-		icon_width = gdk_pixbuf_get_width (priv->favicon);
-		icon_height = gdk_pixbuf_get_height (priv->favicon);
+		icon_width = gdk_pixbuf_get_width (favicon);
+		icon_height = gdk_pixbuf_get_height (favicon);
 	}
 
 	context = gtk_widget_get_pango_context (widget);
@@ -740,13 +764,13 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 
 	pango_layout_get_pixel_size (layout, &layout_width, &layout_height);
 
-	if (priv->favicon != NULL)
+	if (favicon != NULL)
 	{
 		favicon_offset_x = icon_width + (2 * DRAG_ICON_ICON_PADDING);
 	}
 
 	surface_width = layout_width + favicon_offset_x +
-			(DRAG_ICON_LAYOUT_PADDING * 2);
+			(DRAG_ICON_LAYOUT_PADDING * 3);
 	surface_height = MAX (layout_height, icon_height) +
 			(DRAG_ICON_LAYOUT_PADDING * 2);
 
@@ -766,14 +790,14 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 	gdk_cairo_set_source_rgba (cr, &color);
 	cairo_fill (cr);
 
-	if (priv->favicon != NULL)
+	if (favicon != NULL)
 	{
 		double x;
 		double y;
 
 		x = 1 + DRAG_ICON_LAYOUT_PADDING + DRAG_ICON_ICON_PADDING;
-		y = 1 + DRAG_ICON_LAYOUT_PADDING + (surface_height - icon_height) / 2;
-		gdk_cairo_set_source_pixbuf (cr, priv->favicon, x, y);
+		y = (surface_height - icon_height) / 2;
+		gdk_cairo_set_source_pixbuf (cr, favicon, x, y);
 		cairo_rectangle (cr, x, y, icon_width, icon_height);
 		cairo_fill (cr);
 	}
@@ -791,6 +815,7 @@ favicon_create_drag_surface (EphyLocationEntry *entry,
 	g_free (address);
 	g_free (title);
 	g_string_free (text, TRUE);
+	g_clear_object (&favicon);
 
 	return surface;
 }
@@ -1029,6 +1054,18 @@ cursor_on_match_cb  (GtkEntryCompletion *completion,
 	return TRUE;
 }
 
+static char *
+rgba_to_hex (GdkRGBA *color)
+{
+	char *hex;
+
+	hex = g_strdup_printf ("#%04X%04X%04X",
+			       (guint)(color->red * (gdouble)65535),
+			       (guint)(color->green * (gdouble)65535),
+			       (guint)(color->blue * (gdouble)65535));
+	return hex;
+}
+
 static void
 textcell_data_func (GtkCellLayout *cell_layout,
 		    GtkCellRenderer *cell,
@@ -1038,16 +1075,10 @@ textcell_data_func (GtkCellLayout *cell_layout,
 {
 	GtkWidget *entry;
 	EphyLocationEntryPrivate *priv;
-	PangoAttrList *list;
-	PangoAttribute *att;
-
 	char *ctext;
 	char *title;
 	char *url;
-
 	GtkStyleContext *style;
-	GdkRGBA color;
-
 	GValue text = { 0, };
 
 	entry = GTK_WIDGET (data);
@@ -1057,41 +1088,32 @@ textcell_data_func (GtkCellLayout *cell_layout,
 			priv->url_col, &url,
 			-1);
 
-	list = pango_attr_list_new ();
-
 	if (url)
 	{
-		/* Do not show internal ephy-about: protocol to users */
-		if (g_str_has_prefix (url, EPHY_ABOUT_SCHEME)) {
-			g_free (url);
-			url = g_strdup_printf ("about:%s", url + strlen (EPHY_ABOUT_SCHEME) + 1);
-		}
+		GdkRGBA color;
+		char *color_text;
 
-		ctext = g_strdup_printf ("%s\n%s", title, url);
+		sanitize_location (&url);
 
 		style = gtk_widget_get_style_context (entry);
 		gtk_style_context_get_color (style, GTK_STATE_FLAG_INSENSITIVE,
 					     &color);
 
-		att = pango_attr_foreground_new
-		   (color.red * 65535, color.green * 65535, color.blue * 65535);
-		att->start_index = strlen (title)+1;
-		pango_attr_list_insert (list, att);
+		color_text = rgba_to_hex (&color);
+		ctext = g_markup_printf_escaped ("%s\n<span font-size=\"small\" color=\"%s\">%s</span>", title, color_text, url);
+		g_free (color_text);
 	        g_free (title);
 	}
 	else
 	{
 		ctext = title;
 	}
-	g_object_set (cell, "attributes", list, NULL);
 
 	g_value_init (&text, G_TYPE_STRING);
 	g_value_take_string (&text, ctext);
-	g_object_set_property (G_OBJECT (cell), "text", &text);
+	g_object_set_property (G_OBJECT (cell), "markup", &text);
 	g_value_unset (&text);
 
-	pango_attr_list_unref (list);
-
 	g_free (url);
 }
 
@@ -1225,11 +1247,11 @@ ephy_location_entry_set_completion (EphyLocationEntry *entry,
 	 * with anything in the entry.
 	 */
 
-#define ROW_PADDING_VERT 2
+#define ROW_PADDING_VERT 4
 
 #define ICON_PADDING_LEFT 5
 #define ICON_CONTENT_WIDTH 16
-#define ICON_PADDING_RIGHT 2
+#define ICON_PADDING_RIGHT 4
 
 #define ICON_CONTENT_HEIGHT 16
 
@@ -1590,17 +1612,17 @@ ephy_location_entry_set_show_lock (EphyLocationEntry *entry,
 }
 
 /**
- * ephy_location_entry_set_lock_stock:
+ * ephy_location_entry_set_lock_state:
  * @entry: an #EphyLocationEntry widget
- * @stock_id: a stock_id from GTK+ stock icons
+ * @state: the #EphyLocationLockState
  *
  * Set the lock icon to be displayed, to actually show the icon see 
  * ephy_location_entry_set_show_lock.
  *
  **/
 void
-ephy_location_entry_set_lock_stock (EphyLocationEntry *entry,
-				    const char *stock_id)
+ephy_location_entry_set_lock_state (EphyLocationEntry *entry,
+				    EphyLocationLockState state)
 
 {
 	EphyLocationEntryPrivate *priv;
@@ -1618,10 +1640,10 @@ ephy_location_entry_set_lock_stock (EphyLocationEntry *entry,
 	 * their security infrastructure (broken cert, etc). For
 	 * everything else, nothing is shown.
 	 */
-	if (!stock_id || g_str_equal (stock_id, STOCK_LOCK_BROKEN))
-		priv->lock_gicon = g_themed_icon_new_with_default_fallbacks ("channel-insecure-symbolic");
-	else
+	if (state == EPHY_LOCATION_LOCK_STATE_SECURE)
 		priv->lock_gicon = g_themed_icon_new_with_default_fallbacks ("channel-secure-symbolic");
+	else
+		priv->lock_gicon = g_themed_icon_new_with_default_fallbacks ("channel-insecure-symbolic");
 
 	if (priv->show_lock)
 		gtk_entry_set_icon_from_gicon (GTK_ENTRY (entry),
