@@ -102,6 +102,7 @@ update_warning_idle (EphyBookmarkProperties *properties)
 	
 	priv->duplicate_idle = g_timeout_add 
 	  (500, (GSourceFunc)update_warning, properties);
+	g_source_set_name_by_id (priv->duplicate_idle, "[epiphany] update_warning");
 }
 
 static void
@@ -610,7 +611,7 @@ ephy_bookmark_properties_constructor (GType type,
 	text = g_strdup_printf (ngettext("%d _Similar", "%d _Similar", 0), 0);
 	gtk_button_set_label (GTK_BUTTON (priv->warning), text);
 	g_free (text);
-	widget = gtk_image_new_from_icon_name (_("dialog-warning"), GTK_ICON_SIZE_BUTTON);
+	widget = gtk_image_new_from_icon_name ("dialog-warning", GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show (widget);
 	gtk_button_set_image (GTK_BUTTON (priv->warning), widget);
 	g_object_set (priv->warning, "sensitive", FALSE, NULL);
