@@ -28,10 +28,11 @@
 #include "ephy-shell.h"
 
 static void
-test_ephy_completion_model_create ()
+test_ephy_completion_model_create (void)
 {
     EphyCompletionModel *model;
-    model = ephy_completion_model_new ();
+    model = ephy_completion_model_new (EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ())),
+                                       ephy_shell_get_bookmarks (ephy_shell_get_default ()));
     g_assert (model);
     g_object_unref (model);
 }
@@ -51,12 +52,13 @@ update_empty_cb (EphyHistoryService *service,
 }
 
 static void
-test_ephy_completion_model_update_empty ()
+test_ephy_completion_model_update_empty (void)
 {
     EphyCompletionModel *model;
     GMainLoop *loop = NULL;
 
-    model = ephy_completion_model_new ();
+    model = ephy_completion_model_new (EPHY_HISTORY_SERVICE (ephy_embed_shell_get_global_history_service (ephy_embed_shell_get_default ())),
+                                       ephy_shell_get_bookmarks (ephy_shell_get_default ()));
     g_assert (model);
 
     loop = g_main_loop_new (NULL, FALSE);
