@@ -141,8 +141,11 @@ test_ephy_download_new (Fixture *fixture, gconstpointer data)
 static void
 test_ephy_download_new_for_uri (Fixture *fixture, gconstpointer data)
 {
-  g_assert_cmpstr (fixture->source, ==,
-                   ephy_download_get_source_uri (fixture->download));
+  WebKitDownload *download = ephy_download_get_webkit_download (fixture->download);
+  WebKitURIRequest *request = webkit_download_get_request (download);
+
+  g_assert(request);
+  g_assert_cmpstr (fixture->source, ==, webkit_uri_request_get_uri (request));
 }
 
 static void
@@ -152,7 +155,6 @@ test_ephy_download_start (Fixture *fixture, gconstpointer data)
                     G_CALLBACK (completed_cb), fixture);
 
   ephy_download_set_action (fixture->download, EPHY_DOWNLOAD_ACTION_DO_NOTHING);
-  ephy_download_start (fixture->download);
   g_main_loop_run (fixture->loop);
 }
 
@@ -166,7 +168,6 @@ main (int argc, char *argv[])
 
   ephy_debug_init ();
   ephy_embed_prefs_init ();
-  _ephy_shell_create_instance (EPHY_EMBED_SHELL_MODE_TEST);
 
   if (!ephy_file_helpers_init (NULL,
                                EPHY_FILE_HELPERS_PRIVATE_PROFILE | EPHY_FILE_HELPERS_ENSURE_EXISTS,
@@ -175,6 +176,8 @@ main (int argc, char *argv[])
     return -1;
   }
 
+  _ephy_shell_create_instance (EPHY_EMBED_SHELL_MODE_TEST);
+
   server = soup_server_new (SOUP_SERVER_PORT, 0, NULL);
   soup_server_run_async (server);
 
