@@ -29,6 +29,7 @@
 #include "ephy-file-helpers.h"
 #include "ephy-history-service.h"
 #include "ephy-private.h"
+#include "ephy-settings.h"
 #include "ephy-shell.h"
 #include "ephy-web-view.h"
 
@@ -244,12 +245,12 @@ test_ephy_web_view_non_search_regex (void)
   g_regex_unref (regex_domain);
 }
 
-/* FIXME: we hardcode the ddg search for now, since it's the
- * default. */
-static struct {
+typedef struct {
   char *url;
   char *expected;
-} normalize_or_autosearch[] = {
+} normalize_or_autosearch_t;
+
+normalize_or_autosearch_t normalize_or_autosearch_test_ddg[] = {
   { "google.com", "http://google.com" },
   { "http://google.com", "http://google.com" },
   { "http://google.com/this/is/a/path", "http://google.com/this/is/a/path" },
@@ -265,25 +266,56 @@ static struct {
   { "\"a quoted string should be searched\"", "http://duckduckgo.com/?q=%22a+quoted+string+should+be+searched%22&t=epiphany" }
 };
 
+normalize_or_autosearch_t normalize_or_autosearch_test_google[] = {
+  { "search", "http://www.google.com/search?q=search" },
+  { "lala/lala", "http://www.google.com/search/?q=lala%2Flala" },
+};
+
 static void
-test_ephy_web_view_normalize_or_autosearch (void)
+verify_normalize_or_autosearch_urls (EphyWebView *view,
+                                     normalize_or_autosearch_t *test)
 {
   int i;
-  EphyWebView *view;
-  
-  view = EPHY_WEB_VIEW (ephy_web_view_new ());
 
-  for (i = 0; i < G_N_ELEMENTS (normalize_or_autosearch); i++) {
+  for (i = 0; i < G_N_ELEMENTS (test); i++) {
     char *url, *result;
 
-    url = normalize_or_autosearch[i].url;
+    url = test[i].url;
 
     result = ephy_web_view_normalize_or_autosearch_url (view, url);
-    g_assert_cmpstr (result, ==, normalize_or_autosearch[i].expected);
+    g_assert_cmpstr (result, ==, test[i].expected);
 
     g_free (result);
   }
+}
 
+static void
+test_ephy_web_view_normalize_or_autosearch (void)
+{
+  char *default_engine_url;
+  EphyWebView *view;
+  
+  view = EPHY_WEB_VIEW (ephy_web_view_new ());
+  default_engine_url = g_settings_get_string (EPHY_SETTINGS_MAIN,
+                                              EPHY_PREFS_KEYWORD_SEARCH_URL);
+
+  g_settings_set_string (EPHY_SETTINGS_MAIN,
+                         EPHY_PREFS_KEYWORD_SEARCH_URL,
+                         "http://duckduckgo.com/?q=%s&t=epiphany");
+
+  verify_normalize_or_autosearch_urls (view, normalize_or_autosearch_test_ddg);
+
+  g_settings_set_string (EPHY_SETTINGS_MAIN,
+                         EPHY_PREFS_KEYWORD_SEARCH_URL,
+                         "http://www.google.com/?q=%s");
+
+  verify_normalize_or_autosearch_urls (view, normalize_or_autosearch_test_google);
+
+  g_settings_set_string (EPHY_SETTINGS_MAIN,
+                         EPHY_PREFS_KEYWORD_SEARCH_URL,
+                         default_engine_url);
+
+  g_free (default_engine_url);
   g_object_unref (g_object_ref_sink (view));
 }
 
